#ifndef _AIM_NET_THREAD_H_
#define _AIM_NET_THREAD_H_

// for the NetThread base class
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <unistd.h>
#include <malloc.h>
//#include "TCPNetThread.h"
#include "TCPThread.h"
#include "GenList.h"
#include "NetException.h"
#include "AIMDataTypes.h"

#include "constants.h"


// request id type constants
const int RID_GET_INFO = 1;






const uint32 BEAIM_NET_DATA_AVAILABLE = 'dAvB';
const uint32 BEAIM_NET_CONNECTED = 'cnTD';
const uint32 BEAIM_NET_LOST_CONNECTION = 'lScn';




// So we can translate request ids back to user ids
struct userRequestID {
	int type;
	unsigned int reqID;
	char userid[DISPLAY_NAME_MAX];
};


// NetThread derivative that handles AIM communications
class AIMNetThread : public TCPThread {

	public:
		AIMNetThread();
		void MessageReceived( BMessage* );
		//void Close();

		void Login( BMessage* );
		void Logout();
		void CancelSignOn();
		
		void SendMessage( BMessage* );
		void RequestPersonInfo( BMessage* );
		void SearchByEmail( BMessage* );
		void SendArbitraryData( BMessage* );
		void SendIdleMessage( bool );
		void SendIdlePulse();
		
	protected:
		void Connected();
		void Disconnected();
		void DataWaiting();
		//void HandleError( int error, const char* message = 0 );
		
		void LoginStep( const char*, bool=true );
		void LoadBuddyList( DataContainer& buddies );
		
	private:

		// Send and receive functions
		void SendPacket( DataContainer packet, char channel );
		void Transmit( DataContainer packet );
		char Receive();
		void ProcessCommand();
		void SendSNACPacket( SNAC_Object& obj );
		
		// useful functions
		DataContainer ToAIMWord( unsigned short );
		DataContainer ToTLV( unsigned short, unsigned short, DataContainer );
		unsigned short GetWord( char* input );
		unsigned short GetWord( DataContainer input, short i );
		void SendAppMessage( BMessage* );
		bool CompareScreenNames( char* a, char* b );

		// packet decoder functions
		void DecodeSNACPacket( SNAC_Object& snac );
		void DecodeUserSNAC( SNAC_Object& snac );
		void DecodeBuddyListSNAC( SNAC_Object& snac );
		void DecodeMessagingSNAC( SNAC_Object& snac );
		void DecodeAdministrativeSNAC( SNAC_Object& snac );
		void DecodeBOSSNAC( SNAC_Object& snac );
		void DecodeOtherSNAC( SNAC_Object& snac );
		void DecodeGenericServiceSNAC( SNAC_Object& snac );

		// Login functions
		void DoLogin( BMessage* info = NULL );
		void SendLoginData();
		void ParseLoginResponse( DataContainer data );
		DataContainer EncryptPassword( char* password );
		void SendBOSRequest();
		void ReceiveBOSResponse();
		void NowOnline();
		void LoginFailure( char* message );
	
		// "functional" functions
		void DecodeBuddyStuff( DataContainer, short&, BMessage* );		
		void ReceiveMessage( SNAC_Object& snac );
		void BuddyOnline( SNAC_Object& snac );
		void BuddyOffline( SNAC_Object& snac );
		void ReceivePersonInfo( SNAC_Object& snac );
		void FailedPersonInfo( SNAC_Object& snac );
		void UpdateServerBuddyList( BMessage* );
		void SendRemovePersonMessage( char* name, int len );
		void ReceiveSearchResults( SNAC_Object& snac );
		void ReceiveFailedSearch( SNAC_Object& snac );
	
		// Sequence numbering stuff
		unsigned int sendSequence;
		unsigned int recvSequence;
		
		// keeps track of various request ids
		GenList<userRequestID> pendingRequests;
		unsigned int emailSearchID;
		unsigned int requestID;
		
		// connection info data
		int preLoginPhase;
		bool connected;
		
		// temporary local copies of the screen name and password
		char* loginScreenName;
		char* loginPassword;
		
		// The received buffer
		DataContainer recvBuf, cookie;
};

#endif
