#include <ctype.h>
#include "MiscStuff.h"
#include "Prefs.h"
#include "Say.h"

//-------------------------------------------------------------------------

GPreferences::GPreferences() {

	status_t error;
	prefsDirectory = new BDirectory();
	prefsFile = new BFile();
	globalPrefsFile = new BFile();
	strcpy( UserName, "default" );
	
	// Try and open the BeAIM prefs folder, and create a new one if it doesn't exist
	error = prefsDirectory->SetTo( "/boot/home/config/settings/BeAIM" );
	if( error == B_ENTRY_NOT_FOUND )
		prefsDirectory->CreateDirectory( "/boot/home/config/settings/BeAIM", prefsDirectory );

	// Now verify the global prefs file
	error = globalPrefsFile->SetTo( "/boot/home/config/settings/BeAIM/globalprefs", B_READ_WRITE );
	if( error == B_ENTRY_NOT_FOUND )
		prefsDirectory->CreateFile( "/boot/home/config/settings/BeAIM/globalprefs", prefsFile );	

	// make the locker
	lock = new BLocker( "prefs_locker" );

	LoadDefaultUser();
}

//-------------------------------------------------------------------------

GPreferences::~GPreferences() {

	prefsFile->Sync();
	delete prefsFile;
	delete globalPrefsFile;
	delete prefsDirectory;
	delete lock;
}

//-------------------------------------------------------------------------

void GPreferences::LoadUserFile( char* user, bool create ) {
	
	lock->Lock();
	status_t error;

	// Check and see if it's blank
	if( strcmp( user, "" ) == 0 ) {
		LoadDefaultUser();
		return;
	}		

	// save the user name
	strcpy( UserName, user );

	// Make the correct filename string
	char filename[1024];
	MakeFilenameFromUsername( filename, UserName );
	
	// Now verify the prefs file
	error = prefsFile->SetTo( filename, B_READ_WRITE );
	if( error == B_ENTRY_NOT_FOUND && create )
		prefsDirectory->CreateFile( filename, prefsFile );
		
	lock->Unlock();
}

//-------------------------------------------------------------------------

void GPreferences::LoadDefaultUser() {

	lock->Lock();
	status_t error;
	
	// Now verify the prefs file
	error = prefsFile->SetTo( "/boot/home/config/settings/BeAIM/defaultuser", B_READ_WRITE );
	if( error == B_ENTRY_NOT_FOUND )
		prefsDirectory->CreateFile( "/boot/home/config/settings/BeAIM/defaultuser", prefsFile );
		
	lock->Unlock();
}

//-------------------------------------------------------------------------

void GPreferences::WriteBool( char* name, bool value, bool global ) {

	lock->Lock();
	if( global )
		globalPrefsFile->WriteAttr( name, B_BOOL_TYPE, 0, &value, sizeof(bool) );
	else
		prefsFile->WriteAttr( name, B_BOOL_TYPE, 0, &value, sizeof(bool) );
	lock->Unlock();
}

//-------------------------------------------------------------------------

bool GPreferences::ReadBool( char* name, bool def, bool global ) {

	lock->Lock();
	bool ret;
	ssize_t res;
	
	if( global )
		res = globalPrefsFile->ReadAttr( name, B_BOOL_TYPE, 0, &ret, sizeof(bool) );
	else
		res = prefsFile->ReadAttr( name, B_BOOL_TYPE, 0, &ret, sizeof(bool) );
	
	if( res == B_ENTRY_NOT_FOUND )
		ret = def;
		
	lock->Unlock();
	return ret;
}

//-------------------------------------------------------------------------
		
void GPreferences::WriteString( char* name, char* val, bool global ) {

	lock->Lock();
	int gSize;
	int hSize;
	char buf[255];

	strcpy( buf, val );
	gSize = strlen( buf ) + 1;
	
	
	if( global ) {
		globalPrefsFile->RemoveAttr( name );
		hSize = globalPrefsFile->WriteAttr( name, B_STRING_TYPE, 0, buf, gSize );
	} else {
		prefsFile->RemoveAttr( name );
		hSize = prefsFile->WriteAttr( name, B_STRING_TYPE, 0, buf, gSize );
	}
	lock->Unlock();
}

//-------------------------------------------------------------------------
		
void GPreferences::ReadString( char* name, char* buf, char* def, unsigned int bufSize, bool global ) {

	lock->Lock();
	ssize_t res;
	
	if( global )
		res = globalPrefsFile->ReadAttr( name, B_STRING_TYPE, 0, buf, bufSize );
	else
		res = prefsFile->ReadAttr( name, B_STRING_TYPE, 0, buf, bufSize );		
	
	if( res == B_ENTRY_NOT_FOUND )
		strcpy( buf, def );
		
	lock->Unlock();
}

//-------------------------------------------------------------------------
		
void GPreferences::WriteInt32( char* name, int32 value, bool global ) {

	lock->Lock();
	if( global )
		globalPrefsFile->WriteAttr( name, B_INT32_TYPE, 0, &value, sizeof(int32) );
	else
		prefsFile->WriteAttr( name, B_INT32_TYPE, 0, &value, sizeof(int32) );
	lock->Unlock();
}

//-------------------------------------------------------------------------
		
int32 GPreferences::ReadInt32( char* name, int32 def, bool global ) {

	lock->Lock();
	int32 ret;
	ssize_t res;
	
	if( global )
		res = globalPrefsFile->ReadAttr( name, B_INT32_TYPE, 0, &ret, sizeof(int32) );
	else
		res = prefsFile->ReadAttr( name, B_INT32_TYPE, 0, &ret, sizeof(int32) );
	
	if( res == B_ENTRY_NOT_FOUND )
		ret = def;
		
	lock->Unlock();
	return ret;
}

//-------------------------------------------------------------------------

bool GPreferences::GetNextUser( char* user, bool first ) {

	lock->Lock();
	bool keepGoing = true, retval = false;
	char testname[6];
	entry_ref fileRef;
	status_t res;

	// if this is the first one, rewind the directory object
	if( first )
		prefsDirectory->Rewind();
		
	// Loop through all the entries in the directory
	while( keepGoing ) {
	
		// Get the next one and break if it ain't there
		res = prefsDirectory->GetNextRef( &fileRef );		
		if( res == B_ENTRY_NOT_FOUND ) {
			keepGoing = false;
			retval = false;
			break;
		}
		
		// it's now safe to assume the entry exists...
		// see if it's a people file by comparing the first 7 letters
		if( strlen(fileRef.name) > 5 ) {
			strncpy( testname, fileRef.name, 5 );
			testname[5] = '\0';
			
			// OK, found a valid user
			if( strcmp( testname, "user-" ) == 0 ) {

				// load up the user file for that user and get the prefs
				strcpy( user, fileRef.name + 5 );
				LoadUserFile( user );
				ReadString( "formattedname", user, user, strlen(user) );			
				LoadDefaultUser();
				keepGoing = false;
				retval = true;
			}
		}
	}

	lock->Unlock();
	return retval;
}

//-------------------------------------------------------------------------	

void GPreferences::GetUserLoginParameters( char* user, char* pw, int pwlen, 
										   bool& autoLogin, bool& savePassword )
{
	lock->Lock();
	savePassword = false;	
	
	// load up the user's file and get the info
	LoadUserFile( user );
	savePassword = ReadBool( "savepassword", false );
	autoLogin = savePassword && ReadBool( "autologin", false );
	if( savePassword )
		ReadString( "password", pw, "", pwlen );
	LoadDefaultUser();
	
	lock->Unlock();
}

//-------------------------------------------------------------------------	