#include <Window.h>
#include <TabView.h>
#include <Box.h>
#include <stdio.h>
#include "PrefsWindow.h"
#include "constants.h"
#include "Globals.h"
#include "Say.h"

//-------------------------------------------------------------------------------------------------------


GenPrefView::GenPrefView(BRect rect, bool go)
	   	   : BView(rect, "GenPrefsTab", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW )
{
	SetViewColor( 216, 216, 216 );
	globalOnly = go;
	
	// Show Buddy list in all workspaces
	BRect CheckRect = BRect(4,4,20,20);
	BuddyListAllWorkspaces = new BCheckBox( CheckRect, "", "Show the Buddy List in all workspaces",  NULL );
	BuddyListAllWorkspaces->ResizeToPreferred();
	BuddyListAllWorkspaces->SetEnabled( !globalOnly );
	AddChild( BuddyListAllWorkspaces );	
	
	// Play sound effects or not
	CheckRect.top += 20;
	PlaySounds = new BCheckBox( CheckRect, "", "Play sound effects",  NULL );
	PlaySounds->ResizeToPreferred();
	PlaySounds->SetEnabled( !globalOnly );
	AddChild( PlaySounds );

	// enable the BeAIM Deskbar Icon
	CheckRect.top += 20;
	UseDeskbar = new BCheckBox( CheckRect, "", "Enable BeAIM deskbar icon",  NULL );
	UseDeskbar->ResizeToPreferred();
	UseDeskbar->SetEnabled( !globalOnly );
	AddChild( UseDeskbar );

	// enable the BeAIM Deskbar Icon
	CheckRect.top += 20;
	DoIdleTime = new BCheckBox( CheckRect, "", "Report your idle time to the AIM server",  NULL );
	DoIdleTime->ResizeToPreferred();
	DoIdleTime->SetEnabled( !globalOnly );
	AddChild( DoIdleTime );

	// Now initialize them
	BuddyListAllWorkspaces->SetValue( prefs->ReadBool("bl_aw", true) );
	PlaySounds->SetValue( prefs->ReadBool("gm_ps", true) );
	UseDeskbar->SetValue( prefs->ReadBool("gm_dk", true) );
	DoIdleTime->SetValue( prefs->ReadBool("gm_id", true) );
}


void GenPrefView::Save() {

	// save the prefs
	if( !globalOnly ) {
		prefs->WriteBool("bl_aw", BuddyListAllWorkspaces->Value() );
		prefs->WriteBool("gm_ps", PlaySounds->Value() );
		prefs->WriteBool("gm_dk", UseDeskbar->Value() );
		prefs->WriteBool("gm_id", DoIdleTime->Value() );
	}
}

//-------------------------------------------------------------------------------------------------------


ConnectionPrefView::ConnectionPrefView(BRect rect, bool go)
	   	   : BView(rect, "ConPrefsTab", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW )
{
	SetViewColor( 216, 216, 216 );
	BBox* serverBox;
	char hostName[255];
	char portName[50];
	globalOnly = go;

	// Add the server group box
	serverBox = new BBox( BRect(8,4,250,82) );
	serverBox->SetLabel("AIM Server");
	AddChild( serverBox );

	// set up the server text boxes
	conHost = new BTextControl( BRect(15,20,225,40), "hostname", "Host:", "", NULL );
	conHost->SetViewColor( 216, 216, 216 );
	conHost->SetDivider( 50 );
	conPort = new BTextControl( BRect(15,45,225,65), "portname", "Port:", "", NULL );
	conPort->SetViewColor( 216, 216, 216 );
	conPort->SetDivider( 50 );
	
	// add them
	serverBox->AddChild( conHost );
	serverBox->AddChild( conPort );
	
	// now initialize them
	prefs->ReadString( "aim_host", hostName, "login.oscar.aol.com", 255, true );
	conHost->SetText( hostName );
	sprintf( portName, "%lu", prefs->ReadInt32("aim_port", 5190, true) );
	conPort->SetText( portName );
}


void ConnectionPrefView::Save() {

	int32 port = 0;

	prefs->WriteString( "aim_host", (char*)conHost->Text(), true );
	sscanf( (char*)conPort->Text(), "%lu", &port );
	prefs->WriteInt32( "aim_port", port, true );
}


//-------------------------------------------------------------------------------------------------------


ChatWindowPrefView::ChatWindowPrefView(BRect rect, bool go)
	   	   : BView(rect, "ConPrefsTab", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW )
{
	SetViewColor( 216, 216, 216 );
	globalOnly = go;

	// Show chat windows in all workspaces
	BRect CheckRect = BRect(4,4,20,20);
	ChatWindowAllWorkspaces = new BCheckBox( CheckRect, "", "Show chat windows in all workspaces by default",  NULL );
	ChatWindowAllWorkspaces->ResizeToPreferred();
	ChatWindowAllWorkspaces->SetEnabled( !globalOnly );
	AddChild( ChatWindowAllWorkspaces );

	// Popup the chat window when a message is received
	CheckRect.top += 20;
	ChatWindowPopup = new BCheckBox( CheckRect, "", "Popup chat windows when messages are received by default",  NULL );
	ChatWindowPopup->ResizeToPreferred();
	ChatWindowPopup->SetEnabled( !globalOnly );
	AddChild( ChatWindowPopup );

	// Show font color and size attributes in the chat windows
	CheckRect.top += 20;
	ChatFontColorSizes = new BCheckBox( CheckRect, "", "Show font color and size attributes by default",  NULL );
	ChatFontColorSizes->ResizeToPreferred();
	ChatFontColorSizes->SetEnabled( !globalOnly );
	AddChild( ChatFontColorSizes );

	// Show URL/mailto links by default
	CheckRect.top += 20;
	ChatShowLinks = new BCheckBox( CheckRect, "", "Enable web/email links",  NULL );
	ChatShowLinks->ResizeToPreferred();
	ChatShowLinks->SetEnabled( !globalOnly );
	AddChild( ChatShowLinks );
	
	// Prefix incoming messages with '>' in the title bar
	CheckRect.top += 20;
	ChatPrefixNewMessages = new BCheckBox( CheckRect, "", "Prefix incoming messages with '>' in the title bar when inactive",  NULL );
	ChatPrefixNewMessages->ResizeToPreferred();
	ChatPrefixNewMessages->SetEnabled( !globalOnly );
	AddChild( ChatPrefixNewMessages );

	// now initialize them
	ChatWindowAllWorkspaces->SetValue( prefs->ReadBool("cw_aw", false) );
	ChatWindowPopup->SetValue( prefs->ReadBool("cw_pu", false) );
	ChatFontColorSizes->SetValue( prefs->ReadBool("cw_fc", true) );
	ChatShowLinks->SetValue( prefs->ReadBool("cw_lk", true) );
	ChatPrefixNewMessages->SetValue( prefs->ReadBool( "cw_p>", false ) );
}


void ChatWindowPrefView::Save() {

	// save the prefs
	if( !globalOnly ) {
		prefs->WriteBool("cw_aw", ChatWindowAllWorkspaces->Value() );
		prefs->WriteBool("cw_pu", ChatWindowPopup->Value() );
		prefs->WriteBool("cw_fc", ChatFontColorSizes->Value() );
		prefs->WriteBool("cw_lk", ChatShowLinks->Value() );
		prefs->WriteBool("cw_p>", ChatPrefixNewMessages->Value() );
	}
}


//-------------------------------------------------------------------------------------------------------

PrefsWindow::PrefsWindow(BRect frame, bool globalOnly)
			: BWindow(frame, "BeAIM Preferences", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE )
{
	BRect r = Bounds();
	BTab *generaltab, *connectiontab;
	BTabView *tabView; 

	// Add the basic background
	BView* bgView = new BView( Bounds(), "PrefView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE );
	bgView->SetViewColor( 216, 216, 216 );
	AddChild( bgView );

	// Add the tabview control
	r.InsetBy( 4, 4 );
	r.bottom -= 31;
	tabView = new BTabView(r, "tab_view"); 
	tabView->SetViewColor( 216, 216, 216 );
	bgView->AddChild( tabView );
	BRect bottomrect = Bounds();
	bottomrect.InsetBy(4,4);
	bottomrect.top = r.bottom + 4;	
	
	// Add the tabs themselves
	r = tabView->Bounds();
	r.InsetBy(5,5); 
	r.bottom -= tabView->TabHeight(); 

	// General tab
	generaltab = new BTab(); 
	tabView->AddTab( genPrefs = new GenPrefView(r,globalOnly), generaltab ); 
	generaltab->SetLabel("General"); 
	
	// Chat Window prefs tab
	connectiontab = new BTab(); 
	tabView->AddTab( chatPrefs = new ChatWindowPrefView(r,globalOnly), connectiontab );
	connectiontab->SetLabel("Chat windows");
	
	// Connection prefs tab
	connectiontab = new BTab(); 
	tabView->AddTab( conPrefs = new ConnectionPrefView(r,globalOnly), connectiontab );
	connectiontab->SetLabel("Connection");	
	
	//Add buttons
	BRect buttonrect = bottomrect;
	buttonrect.top++;
	buttonrect.left = buttonrect.right - 60;
	BButton *btnCancel = new BButton(buttonrect, "Cancel", "Cancel", new BMessage(PREFS_CANCEL), B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);
	buttonrect.right = buttonrect.left - 8;
	buttonrect.left = buttonrect.right - 60;
	BButton *btnSave = new BButton(buttonrect, "Save", "Save", new BMessage(PREFS_SAVE), B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);	
	btnSave->MakeDefault( true );
	bgView->AddChild(btnSave);
	bgView->AddChild(btnCancel);	

	// Show the view
	tabView->Select( 0 );
	Show();
}


void PrefsWindow::MessageReceived( BMessage* message ) {

	switch( message->what ) {
	
		case PREFS_SAVE:

			// have all the views save themselves
			genPrefs->Save();
			conPrefs->Save();
			chatPrefs->Save();
			
			// send a message to all windows telling them to reload settings
			windows->BroadcastMessage( new BMessage(BEAIM_RELOAD_PREF_SETTINGS) );

			// ... fall thru to Close();

		case B_CANCEL:
		case PREFS_CANCEL:
			Close();
			break;
			
		default:
			BWindow::MessageReceived( message );
	}
}



bool PrefsWindow::QuitRequested()
{
	return(true);
}


// the cancel function
void PrefsWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	// if it's a cancel key, post a B_CANCEL message
	if( msg->what == B_KEY_DOWN )
		if( msg->FindString("bytes")[0] == B_ESCAPE ) {
			PostMessage( new BMessage(B_CANCEL) );
			return;
		}
	
	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}