#include <Screen.h>
#include <SupportKit.h>
#include <Application.h>
#include "WindowManager.h"
#include "ChatWindow.h"
#include "PrefsWindow.h"
#include "InfoWindow.h"
#include "GenericInput.h"
#include "MiscStuff.h"
#include "Say.h"

//---------------------------------------------------------------------

WindowManager::WindowManager() {
	
	IMWindowRect = BRect( 25, 25, 400, 400 );
	lock = new BLocker( "window_lock" );
	resWin = NULL;
	bList = NULL;
	signOn = NULL;
}

//---------------------------------------------------------------------

void WindowManager::ToggleHidden() {

	lock->Lock();

	// toggle the minimization of the Buddy List window...
	if( bList && bList->Lock() ) {
		bList->Minimize(!bList->IsMinimized());
		bList->Unlock();
	}

	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::CalcNewIMRect() {

	lock->Lock();
	BScreen* screen = new BScreen;
	
	// offset it... put it back to top-left if it doesn't fit onscreen
	IMWindowRect.OffsetBy( 20, 20 );
	if( !screen->Frame().Contains(IMWindowRect) )
		IMWindowRect = BRect( 25, 25, 400, 400 );
	
	delete screen;
	lock->Unlock();
}

//---------------------------------------------------------------------

// for DEBUG USE ONLY!!!!!!!
void WindowManager::MakeDataSenderWindow() {

	BRect frame(0,0,415,250);
	frame.OffsetBy( 100, 100 );

	DataSenderWindow* dSend = new DataSenderWindow( frame );
	dSend->Show();
}

//---------------------------------------------------------------------

void WindowManager::UserWindowClosed( BMessage* message )
{
	lock->Lock();

	char ScreenName[INTERNAL_NAME_MAX];
	strcpy( ScreenName, message->FindString("userid") );
	unsigned int proto = (unsigned)message->FindInt32("proto");
	int wtype = (int)message->FindInt32("wtype");
	bool Found = false;
	unsigned int i;
	
	// Find the window in the list
	for( i = 0; i < IMWindows.Count(); ++i ) {
		if( IMWindows[i].type == wtype && IMWindows[i].proto == proto &&
			//strcasecmp(ScreenName, IMWindows[i].name) == 0 )
			CompareScreenNames(ScreenName, IMWindows[i].name) )
		{
			Found = true;
			break;
		}
	}
	
	// Delete the record (if it was there)
	if( Found )
		IMWindows.Delete( i );
		
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::SwitchUserWindow( BMessage* message ) {

	lock->Lock();
	IMWindowItem item, target;
	bool Found = false, keepGoing;
	
	// true if we should go the the next one, false if to the prev
	bool forward = bool(message->what == BEAIM_NEXT_CHAT_WINDOW);
	
	// if the list is empty, just ignore it
	if( IMWindows.IsEmpty() )
		return;	

	// was this sent from a user window or somewhere else?
	if( message->HasString("userid") ) {

		char ScreenName[INTERNAL_NAME_MAX];
		strcpy( ScreenName, message->FindString("userid") );
		unsigned int proto = (unsigned)message->FindInt32("proto");
		int wtype = (int)message->FindInt32("wtype");

		// Find the window in the list
		keepGoing = IMWindows.First( item );
		while( keepGoing ) {
			if( item.type == wtype && item.proto == proto &&
				CompareScreenNames( ScreenName, item.name ) )
			{
				Found = true;
				break;
			}
			keepGoing = IMWindows.Next( item );
		}
	}
	
	// if it wasn't there, use the first or last one
	if( !Found ) {
		if( forward )
			IMWindows.First( target );
		else
			IMWindows.Last( target );
		target.window->Activate();

		lock->Unlock();
		return;
	}

	// now look for the next (or previous) window
	if( forward ) {
		keepGoing = IMWindows.Next( target );
		if( !keepGoing )
			keepGoing = IMWindows.First( target );
	} else {
		keepGoing = IMWindows.Prev( target );
		if( !keepGoing )
			keepGoing = IMWindows.Last( target );
	}

	// activate the target window
	target.window->Activate();

	lock->Unlock();
}

//---------------------------------------------------------------------


void WindowManager::OpenBuddyList( BPoint* point ) {
 
	lock->Lock();
	BScreen* screen = new BScreen;
	BRect frame(0,0,158,340);
	
	frame.OffsetBy( screen->Frame().Width() - frame.Width() - 10, 29 );
	bList = new BuddyList( frame );
	bList->Show();

	delete screen;
	lock->Unlock();	
}

//---------------------------------------------------------------------

void WindowManager::UserWindowOpened( BMessage* message )
{
	lock->Lock();

	BWindow* newWindow;
	char name[INTERNAL_NAME_MAX];

	// Get the data from the BMessage and make a new record out of it
	message->FindPointer( "new_window", (void**)&newWindow );
	strcpy( name, message->FindString("userid") );
	IMWindowItem newRecord( (unsigned)message->FindInt32("proto"), name,
						    (int)message->FindInt32("wtype"), newWindow );
	
	// Add that record to the list
	IMWindows.Add( newRecord );
	
	lock->Unlock();
}

//---------------------------------------------------------------------

BWindow* WindowManager::OpenUserWindow( BMessage* message )
{
	lock->Lock();
	BWindow *userWindow = NULL, *posWindow = NULL;

	char actualName[INTERNAL_NAME_MAX];
	char displayName[DISPLAY_NAME_MAX];
	unsigned int proto = (unsigned)message->FindInt32("proto");
	strcpy( actualName, message->FindString("userid") );
	int wtype = (int)message->FindInt32("wtype");
	if( message->HasPointer("poswindow") )
		message->FindPointer( "poswindow", (void**)&posWindow );
	
	comm->TranslatePersonToDisplay( proto, actualName, displayName );

	for( unsigned int i = 0; i < IMWindows.Count(); ++i ) {

		// The window the user wants is already open... activate it
		if( IMWindows[i].type == wtype && IMWindows[i].proto == proto && 
			//strcasecmp(actualName, IMWindows[i].name) == 0 )
			CompareScreenNames(actualName, IMWindows[i].name) )
		{
			BWindow* theWindow = IMWindows[i].window;
			theWindow->Activate();
			lock->Unlock();
			return theWindow;
		}
	}

	// Otherwise, make a new one
	switch( wtype ) {

		// a message (IM) window
		case USER_MESSAGE_TYPE:
			userWindow = MakeMessageWindow( proto, actualName, displayName );
			break;
			
		case USER_INFO_TYPE:
			userWindow = MakeInfoWindow( proto, actualName, displayName, posWindow );
			break;
	}

	// show it and quit
	userWindow->Show();
	lock->Unlock();
	return userWindow;
}

//---------------------------------------------------------------------

BWindow* WindowManager::MakeMessageWindow( unsigned int proto, char* actualName, char* displayName ) {

	BWindow* ret = new ChatWindow( proto, actualName, displayName, IMWindowRect );
	CalcNewIMRect();
	return ret;
}

//---------------------------------------------------------------------

BWindow* WindowManager::MakeInfoWindow( unsigned int proto, char* actualName, char* displayName,
										BWindow* posWindow )
{
	lock->Lock();
	BRect rect(0,0,330,253);

	// position it appropriately
	if( posWindow )
		MakeDialogFrame( rect, posWindow );
	else
		CenterFrame( rect );

	// make it	
	InfoWindow* ret = new InfoWindow( rect, proto, actualName, displayName );
	lock->Unlock();
	return ret;
}
		
//---------------------------------------------------------------------

void WindowManager::OpenSignOnWindow( BPoint* point ) {

	lock->Lock();
	BRect frame( 0, 0, 200, 205 );
	BScreen* screen = new BScreen;
	
	// create the signon window and show it
	frame.OffsetBy( screen->Frame().Width() - frame.Width() - 10, 29 );
	signOn = new LoginWindow( frame );
	signOn->Show();
	
	delete screen;
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::CloseSignOnWindow( BPoint* point ) {

	lock->Lock();

	// lock the sign on window and close it
	signOn->Lock();
	signOn->Quit();
	
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::ForwardIncomingMessage( BMessage* message ) {

	lock->Lock();

	char ScreenName[INTERNAL_NAME_MAX];
	strcpy( ScreenName, message->FindString("userid") );
	unsigned int proto = (unsigned)message->FindInt32("proto");
	bool Found = false;
	unsigned int i;
	int wtype = (int)message->FindInt32("wtype");
	
	// Find the window in the list
	for( i = 0; i < IMWindows.Count(); ++i ) {
		if( IMWindows[i].type == wtype && IMWindows[i].proto == proto &&
			//strcasecmp(ScreenName, IMWindows[i].name) == 0 )
			CompareScreenNames(ScreenName, IMWindows[i].name) )
		{
			Found = true;
			break;
		}
	}
	
	// Construct a BMessenger and forward the message
	if( Found ) {		
		BMessenger sender( IMWindows[i].window );
		sender.SendMessage( message );
	} else {
		
		BWindow* newWindow = OpenUserWindow( message );
		BMessenger sender( newWindow );
		sender.SendMessage( message );	
	}
	
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::OpenPrefsWindow( bool globalOnly, BPoint* point ) {

	lock->Lock();
	BRect prefsRect( 0, 0, 340, 200 );

	if( bList )
		MakeDialogFrame( prefsRect, bList->Frame().LeftTop() );
	else
		CenterFrame( prefsRect );

	if( point )
		prefsRect.OffsetTo( *point );

	// make the window
	new PrefsWindow( prefsRect, globalOnly );

	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::SendBuddyListMessage( BMessage* message ) {

	lock->Lock();
	bList->PostMessage( message );
	lock->Unlock();
}
		
//---------------------------------------------------------------------		

void WindowManager::SendSignOnMessage( BMessage* message ) {

	lock->Lock();
	signOn->PostMessage( message );
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::BroadcastMessage( BMessage* message ) {

	lock->Lock();
	SendBuddyListMessage( message );
	be_app->PostMessage( message );
	
	// send to all the open chat windows
	for( unsigned int i = 0; i < IMWindows.Count(); ++i )
		IMWindows[i].window->PostMessage( message );
		
	lock->Unlock();
}
		
//---------------------------------------------------------------------

void WindowManager::DisplayMessage( char* message ) {

	lock->Lock();
	(new BAlert("", message, "OK"))->Go();
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::DisplayError( char* message ) {

	// for now, errors and messages are the same thing
	lock->Lock();
	DisplayMessage( message );
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::CenterFrame( BRect& frame ) {

	lock->Lock();
	BRect centered;
	BScreen* screen = new BScreen;	
	
	centered.left = (screen->Frame().Width() - frame.Width()) / 2;
	centered.top = (screen->Frame().Height() - frame.Height()) / 2;
	centered.right = centered.left + frame.Width();
	centered.bottom = centered.top + frame.Height();
	frame = centered;
	
	delete screen;
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::MakeDialogFrame( BRect& frame, BWindow* window ) {
	
	lock->Lock();
	BPoint point;

	// make sure there IS a window	
	if( !window ) {
		lock->Unlock();
		return;
	}
	
	// get it's top left corner and call the real function
	point = window->Frame().LeftTop();
	MakeDialogFrame( frame, point );
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::MakeDialogFrame( BRect& frame, BPoint point ) {

	lock->Lock();
	BScreen* screen = new BScreen;
	float offx = 0, offy = 0;	
	
	// Move the rectangle sligtly down and right relative to point
	frame.OffsetTo( point.x + 20, point.y + 40 );

	// if the the rect isn't totally onscreen
	if( !screen->Frame().Contains( frame ) ) {

		// offset by the amount that the box would be offscreen
		offx = frame.left + frame.Width() - screen->Frame().Width();
		offy = frame.top + frame.Height() - screen->Frame().Height();
		frame.OffsetBy( offx < 0 ? 0 : -offx - 20, offy < 0 ? 0 : -offy - 20 );
	}	
		
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::OpenInputWindow( char* title, char* label, BMessage* msg, BWindow* window,
									 char* initial, int maxLen, bool toApp, char* fname )
{
	lock->Lock();

	// put the rect to window in the right spot
	BRect frame(0,0,200,100);
	if( window )
		MakeDialogFrame( frame, window );
	else
		CenterFrame( frame );
		
	// make the window
	GenericInputWindow* msgBox = new GenericInputWindow( frame, title, label, msg, window,
														 initial, maxLen, fname );

	// check and see if the message is supposed to go the the app (not a window)
	if( toApp )
		msgBox->SetTarget( APP_SIGNATURE );

	msgBox->Show();	
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::OpenEmailSearchResultsWindow( BMessage* message ) {

	lock->Lock();
	BRect resRect( 0, 0, 320, 170 );

	// only make a new results window if it's not already open
	if( !resWin ) {
	
		// put the frame in the right spot
		if( bList )
			MakeDialogFrame( resRect, bList->Frame().LeftTop() );
		else
			CenterFrame( resRect );

		// open the window
		resWin = new SearchResultsWindow( resRect );
		resWin->Show();
	}
	
	// set (or reset) the window
	resWin->Activate();	
	resWin->PostMessage( message );
	
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::ForwardEmailSearchResults( BMessage* message ) {
	if( resWin )
		resWin->PostMessage( message );
}

//---------------------------------------------------------------------

void WindowManager::EmailSearchResultsWindowClosed() {
	resWin = NULL;
}

//---------------------------------------------------------------------